@@ -2,9 +2,9 @@ import logging
 import os
 from aperturedb import CSVParser
 
-PROPERTIES  = "properties"
+PROPERTIES = "properties"
 CONSTRAINTS = "constraints"
-BLOB_PATH   = "filename"
+BLOB_PATH = "filename"
 
 logger = logging.getLogger(__name__)
 
@@ -12,10 +12,10 @@ logger = logging.getLogger(__name__)
 class BlobDataCSV(CSVParser.CSVParser):
     """**ApertureDB Blob Data.**
 
-    This class loads the Blob Data which is present in a csv file,
-    and converts it into a series of aperturedb queries.
+    This class loads the Blob Data which is present in a CSV file,
+    and converts it into a series of ApertureDB queries.
 
-    :::note Is backed by a csv file with the following columns:
+    :::note Is backed by a CSV file with the following columns:
     ``FILENAME``, ``PROP_NAME_1``, ... ``PROP_NAME_N``, ``constraint_PROP_NAME_1``
     :::
 
@@ -25,7 +25,7 @@ class BlobDataCSV(CSVParser.CSVParser):
 
     **constraint_PROP_NAME_1**: Constraints against specific property, used for conditionally adding a Blob.
 
-    Example csv file::
+    Example CSV file::
 
         filename,name,lastname,age,id,constraint_id
         /mnt/blob1,John,Salchi,69,321423532,321423532
@@ -48,7 +48,7 @@ class BlobDataCSV(CSVParser.CSVParser):
     :::
     """
 
-    def __init__(self, filename, **kwargs):
+    def __init__(self, filename: str, **kwargs):
 
         super().__init__(filename, **kwargs)
 
@@ -66,7 +66,6 @@ class BlobDataCSV(CSVParser.CSVParser):
         }
 
     def getitem(self, idx):
-        idx = self.df.index.start + idx
         filename = os.path.join(self.relative_path_prefix,
                                 self.df.loc[idx, BLOB_PATH])
         blob_ok, blob = self.load_blob(filename)
