@@ -4,7 +4,7 @@ from dataclasses import dataclass
 @dataclass(repr=False)
 class Configuration:
     """
-    **Configuration object for aperturedb sdk to be able to connect to ApertureDB**
+    **Configuration object for ApertureDB sdk to be able to connect to ApertureDB**
     """
     host: str
     port: int
@@ -14,10 +14,10 @@ class Configuration:
     use_ssl: bool = True
     use_rest: bool = False
     use_keepalive: bool = True
-    retry_connect_interval_seconds: int = 1
+    retry_interval_seconds: int = 1
     # Max number of attempts to retry the initial connection (0 means infinite)
     # This is useful when the aperturedb server is not ready yet.
-    retry_connect_max_attempts: int = 3
+    retry_max_attempts: int = 3
 
     def __repr__(self) -> str:
         mode = "REST" if self.use_rest else "TCP"
