@@ -4,21 +4,21 @@ import logging
 
 logger = logging.getLogger(__name__)
 ENTITY_CLASS = "EntityClass"
-PROPERTIES   = "properties"
-CONSTRAINTS  = "constraints"
+PROPERTIES = "properties"
+CONSTRAINTS = "constraints"
 
 
 class EntityDataCSV(CSVParser.CSVParser):
     """**ApertureDB Entity Data.**
 
-    This class loads the Entity Data which is present in a csv file,
-    and converts it into a series of aperturedb queries.
+    This class loads the Entity Data which is present in a CSV file,
+    and converts it into a series of ApertureDB queries.
 
-    :::note Is backed by a csv file with the following columns:
+    :::note Is backed by a CSV file with the following columns:
     ``EntityClass``, ``PROP_NAME_1``, ... ``PROP_NAME_N``, ``constraint_PROP1``
     :::
 
-    Example csv file::
+    Example CSV file::
 
         EntityClass,name,lastname,age,id,constraint_id
         Person,John,Salchi,69,321423532,321423532
@@ -42,7 +42,7 @@ class EntityDataCSV(CSVParser.CSVParser):
 
     """
 
-    def __init__(self, filename, **kwargs):
+    def __init__(self, filename: str, **kwargs):
         super().__init__(filename, **kwargs)
 
         self.props_keys = [x for x in self.header[1:]
@@ -63,7 +63,7 @@ class EntityDataCSV(CSVParser.CSVParser):
         eclass = self.df.loc[idx, ENTITY_CLASS]
         q = []
         ae = self._basic_command(idx,
-                                 custom_fields = {
+                                 custom_fields={
                                      "class": eclass
                                  })
 
@@ -80,16 +80,16 @@ class EntityDataCSV(CSVParser.CSVParser):
 class EntityDeleteDataCSV(CSVParser.CSVParser):
     """**ApertureDB Entity Delete Data.**
 
-    This class loads the Entity Data which is present in a csv file,
-    and converts it into a series of aperturedb deletes.
+    This class loads the Entity Data which is present in a CSV file,
+    and converts it into a series of ApertureDB deletes.
 
     :::note::
-    Expects a csv file with the following columns:
+    Expects a CSV file with the following columns:
 
         ``constraint_PROP1``
     :::
 
-    Example csv file::
+    Example CSV file::
 
         constraint_id
         321423532
@@ -122,7 +122,7 @@ class EntityDeleteDataCSV(CSVParser.CSVParser):
         self.command = "Delete" + entity_class
         self.constraint_keyword = "constraints"
         if not use_dask:
-            self.constraint_keys       = [x for x in self.header[0:]]
+            self.constraint_keys = [x for x in self.header[0:]]
 
     def getitem(self, idx):
         idx = self.df.index.start + idx
