@@ -1,5 +1,9 @@
+"""
+**Image Objects**
+"""
+
 from __future__ import annotations
-from typing import Tuple, Union
+from typing import Any, Dict, Iterable, List, Tuple, Union
 import cv2
 import math
 import numpy as np
@@ -19,26 +23,116 @@ import logging
 logger = logging.getLogger(__name__)
 
 
-def np_arr_img_to_bytes(arr, format='JPEG') -> bytes:
+def np_arr_img_to_bytes(arr, format: str = 'JPEG') -> bytes:
+    """
+    **Convert a NumPy array to bytes**
+
+    Args:
+        arr: The NumPy array to convert
+        format: The format of the image. Defaults to "JPEG".
+
+    Returns:
+        bytes: The image as bytes
+    """
     return image_to_bytes(Image.fromarray(arr, 'RGB'), format=format)
 
 
-def image_to_bytes(image: Image, format="JPEG") -> bytes:
+def image_to_bytes(image: Image, format: str = "JPEG") -> bytes:
+    """
+    **Convert an image to bytes**
+
+    Args:
+        image (PIL.Image): The image to convert
+        format (str, optional): The format of the image. Defaults to "JPEG".
+
+    Returns:
+        bytes: The image as bytes
+    """
     with BytesIO() as bytes:
         image.save(bytes, format=format)
         return bytes.getvalue()
 
 
+def rotate(points, angle, c_x=0, c_y=0):
+    """
+    **Rotate a set of points around a center**
+
+    Args:
+        points: The points to rotate; iterable of (x, y) pairs
+        angle: The angle of counterclockwise rotation in degrees
+        c_x: The x coordinate of the center of rotation
+        c_y: The y coordinate of the center of rotation
+
+    Returns:
+        points: The rotated points as a NumPy array of shape (n,2) and type int
+    """
+    ANGLE = np.deg2rad(angle)
+    return np.array(
+        [
+            [
+                c_x + np.cos(ANGLE) * (px - c_x) - np.sin(ANGLE) * (py - c_y),
+                c_y + np.sin(ANGLE) * (px - c_x) + np.cos(ANGLE) * (py - c_y)
+            ]
+            for px, py in points
+        ]
+    ).astype(int)
+
+
+def resolve(points: np.array, image_meta, operations) -> np.array:
+    """
+    **Resolve the coordinates of a bounding box to the original image size**
+
+    Given an array of (x,y) points with respect to the coordinates of an image, returns the corresponding coordinates with respect to a the resized or rotated image that resulted from a series of operations.
+
+    Args:
+        points (NumPy array): The coordinates of the bounding box; shape (...,2)
+        image (dict): The image properties
+        operations (list): The operations applied to the image
+
+    Returns:
+        resolved_points (NumPy array): The resolved coordinates, same shape as points
+    """
+    resolved = points.copy()
+    if image_meta["adb_image_width"] and image_meta["adb_image_height"]:
+        image_meta_width = image_meta["adb_image_width"]
+        image_meta_height = image_meta["adb_image_height"]
+        # print(f"resolve: {resolved}")
+        for operation in operations:
+            if operation["type"] == "resize":
+                x_ratio = operation["width"] / image_meta_width
+                y_ratio = operation["height"] / image_meta_height
+                np.multiply(resolved, [x_ratio, y_ratio],
+                            out=resolved, casting="unsafe")
+                image_meta_width = operation["width"]
+                image_meta_height = operation["height"]
+            if operation["type"] == "rotate":
+                angle = operation["angle"]
+                resolved = rotate(
+                    resolved, angle, c_x=image_meta_width / 2, c_y=image_meta_height / 2)
+            # print(f"resolve: {resolved}")
+    else:
+        logger.warn(
+            "Cannot resolve bounding box coordinates, missing image metadata.")
+        logger.warn(
+            "Reingest the image with image_properties transformer for this functionality.")
+    return resolved
+
+
 class Images(Entities):
     """
-    **The object mapper representation of images in ApertureDB.**
+    **The python wrapper of images in ApertureDB.**
 
-    This class is a layer on top of the native query.
-    It facilitate interactions with images in the database in pythonic way.
+    This class serves 2 purposes:
+    ***This class is a layer on top of the native query.***
+    It facilitate interactions with images in the database in a pythonic way.
+    Abstracts the complexity of the query language and the communication with the database.
 
-    It abstracts the need to batch the responses, and includes utility methods to
-    interconvert the representation into numpy matrices, and find similar images,
+    ***It includes utility methods to visualize image and annotations**
+    Inter convert the representation into NumPy matrices and find similar images,
     related bounding boxes, etc.
+
+    Args:
+        db: The database connector, perhaps as returned by `Utils.create_connector`
     """
     db_object = "_Image"
 
@@ -98,18 +192,19 @@ class Images(Entities):
 
         self.db_connector = db
 
-        self.images          = {}
-        self.images_ids      = []
-        self.image_sizes     = []
-        self.images_bboxes   = {}
+        self.images = {}
+        self.images_ids = []
+        self.image_sizes = []
+        self.images_bboxes = {}
         self.images_polygons = {}
-
         self.overlays = []
+        self.color_for_tag = {}
 
         self.constraints = None
-        self.operations  = None
-        self.format      = None
-        self.limit       = None
+        self.operations = None
+        self.format = None
+        self.limit = None
+        self.query = None
 
         self.adjacent = {}
 
@@ -117,7 +212,7 @@ class Images(Entities):
         self.total_cached_images = 0
         self.display_limit = 20
 
-        self.img_id_prop     = "_uniqueid"
+        self.img_id_prop = "_uniqueid"
         self.bbox_label_prop = "_label"
         self.get_image = True
 
@@ -125,6 +220,20 @@ class Images(Entities):
             self.images_ids = list(
                 map(lambda x: x[self.img_id_prop], response))
 
+        # Blobs can be passed in addition the response.
+        # This would mean that the images are already retrieved.
+        # This is useful for usage of the class dor it's utility methods.
+        if "blobs" in kwargs:
+            blobs = kwargs["blobs"]
+            for i, id in enumerate(self.images_ids):
+                self.images[id] = blobs[i]
+
+        # If the query is passed, we should save the information
+        # for things like operations, constraints, etc.
+        # This is useful for resolving the coordinates of bounding boxes.
+        if "query" in kwargs:
+            self.query = kwargs["query"]
+
     def __retrieve_batch(self, index):
         '''
         **Retrieve the batch that contains the image with the specified index**
@@ -134,10 +243,10 @@ class Images(Entities):
         # for that batch
 
         total_batches = math.ceil(len(self.images_ids) / self.batch_size)
-        batch_id      = int(math.floor(index / self.batch_size))
+        batch_id = int(math.floor(index / self.batch_size))
 
         start = batch_id * self.batch_size
-        end   = min(start + self.batch_size, len(self.images_ids))
+        end = min(start + self.batch_size, len(self.images_ids))
 
         query = []
 
@@ -148,6 +257,7 @@ class Images(Entities):
                     "constraints": {
                         self.img_id_prop: ["==", self.images_ids[idx]]
                     },
+                    "blobs": True
                 }
             }
 
@@ -184,6 +294,9 @@ class Images(Entities):
                     self.img_id_prop: ["==", uniqueid]
                 },
                 "blobs": False,
+                "results": {
+                    "list": ["adb_image_width", "adb_image_height"]
+                },
             }
         }, {
             "FindPolygon": {
@@ -217,30 +330,57 @@ class Images(Entities):
             res, _ = self.db_connector.query(query)
 
             polygons = []
-            bounds   = []
-            tags     = []
+            bounds = []
+            tags = []
+            meta = []
             polys = res[1]["FindPolygon"]["entities"]
+            operations = self.query["operations"] if self.query and "operations" in self.query else [
+            ]
             for poly in polys:
                 if tag_key and tag_format:
                     tag = tag_format.format(poly[tag_key])
                     tags.append(tag)
+                    meta.append(res[0]["FindImage"]["entities"][0])
+
                 bounds.append(poly["_bounds"])
-                polygons.append(poly["_vertices"])
+                converted = []
+                for vert in poly["_vertices"]:
+                    v = resolve(
+                        np.array(vert),
+                        res[0]["FindImage"]["entities"][0],
+                        operations)
+                    converted.append(v)
+                polygons.append(converted)
 
             self.images_polygons[str(uniqueid)] = {
                 "bounds": bounds,
                 "polygons": polygons,
                 "tags": tags,
+                "meta": meta
             }
 
-        except:
+        except Exception as e:
             self.images_polygons[str(uniqueid)] = {
                 "bounds": [],
                 "polygons": [],
-                "tags": []
+                "tags": [],
+                "meta": []
             }
+            logger.warn(
+                f"Cannot retrieve polygons for image {uniqueid}", exc_info=True)
 
-    def __retrieve_bounding_boxes(self, index):
+    def __get_box_from_coords(self, coordinates):
+        box = np.array([
+            (coordinates["x"], coordinates["y"]),
+            (coordinates["x"] +
+             coordinates["width"], coordinates["y"]),
+            (coordinates["x"] + coordinates["width"],
+             coordinates["y"] + coordinates["height"]),
+            (coordinates["x"], coordinates["y"] + coordinates["height"])]
+        )
+        return box
+
+    def __retrieve_bounding_boxes(self, index, constraints):
         # We should fetch all bounding boxes incrementally.
         if self.images_bboxes is None:
             self.images_bboxes = {}
@@ -257,6 +397,9 @@ class Images(Entities):
                 "constraints": {
                     self.img_id_prop: ["==", uniqueid]
                 },
+                "results": {
+                    "list": ["adb_image_width", "adb_image_height"]
+                },
                 "blobs": False,
             }
         }, {
@@ -268,24 +411,41 @@ class Images(Entities):
                 "labels": True
             }
         }]
+        fbbq = query[1]["FindBoundingBox"]
+        if constraints and constraints.constraints:
+            fbbq["constraints"] = constraints.constraints
 
+        uniqueid_str = str(uniqueid)
+        self.images_bboxes[uniqueid_str] = {}
         try:
             res, images = self.db_connector.query(query)
-
             bboxes = []
-            tags   = []
-            for bbox in res[1]["FindBoundingBox"]["entities"]:
-                bboxes.append(bbox["_coordinates"])
-                tags.append(bbox[self.bbox_label_prop])
-
-            uniqueid_str = str(uniqueid)
-            self.images_bboxes[uniqueid_str] = {}
+            tags = []
+            meta = []
+            bounds = []
+            if "entities" in res[1]["FindBoundingBox"]:
+                for bbox in res[1]["FindBoundingBox"]["entities"]:
+                    coordinates = bbox["_coordinates"]
+                    box = self.__get_box_from_coords(coordinates)
+                    operations = self.query["operations"] if self.query and "operations" in self.query else [
+                    ]
+                    resolved = resolve(
+                        box,
+                        # image to bb is 1:n relation
+                        res[0]["FindImage"]["entities"][0],
+                        operations)
+                    bboxes.append(resolved)
+                    tags.append(bbox[self.bbox_label_prop])
+                    meta.append(res[0]["FindImage"]["entities"][0])
+                    bounds.append(box)
+        except Exception as e:
+            logger.warn(
+                f"Cannot retrieve bounding boxes for image {uniqueid}", exc_info=True)
+        finally:
             self.images_bboxes[uniqueid_str]["bboxes"] = bboxes
-            self.images_bboxes[uniqueid_str]["tags"]   = tags
-
-        except:
-            print(self.db_connector.get_last_response_str())
-            raise
+            self.images_bboxes[uniqueid_str]["tags"] = tags
+            self.images_bboxes[uniqueid_str]["meta"] = meta
+            self.images_bboxes[uniqueid_str]["bounds"] = bounds
 
     def total_results(self) -> int:
         """
@@ -298,7 +458,7 @@ class Images(Entities):
         return len(self.images_ids)
 
     def get_image_by_index(self, index: int):
-        """**Get a single image by it's index in the array of retrieved ids**
+        """**Get a single image by its index in the array of retrieved ids**
 
         Args:
             index (int): Position in the image ids retrieved.
@@ -321,7 +481,7 @@ class Images(Entities):
         return self.images[str(uniqueid)]
 
     def get_np_image_by_index(self, index: int):
-        """**Retrieves the numpy representation of image from database**
+        """**Retrieves the NumPy representation of image from database**
 
         Args:
             index (int): Position in the image ids retrieved.
@@ -349,7 +509,7 @@ class Images(Entities):
         """
         if not self.images_bboxes or not str(self.images_ids[index]) in self.images_bboxes:
             # Fetch when not present in the map.
-            self.__retrieve_bounding_boxes(index)
+            self.__retrieve_bounding_boxes(index, None)
 
         try:
             bboxes = self.images_bboxes[str(self.images_ids[index])]
@@ -373,17 +533,17 @@ class Images(Entities):
         """
 
         self.constraints = constraints
-        self.operations  = operations
-        self.format      = format
-        self.limit       = limit
+        self.operations = operations
+        self.format = format
+        self.limit = limit
 
         self.images = {}
         self.images_ids = []
         self.images_sizes = []
         self.images_bboxes = {}
         self.images_polygons = {}
-
         self.overlays = []
+        self.color_for_tag = {}
 
         query = {"FindImage": {}}
 
@@ -444,11 +604,11 @@ class Images(Entities):
             "alpha": alpha
         })
 
-    def add_bbox_overlay(self, polygons, color=None):
+    def add_bbox_overlay(self, bbox, color=None):
         if not color:
             color = self.__random_color()
         self.overlays.append({
-            "bbox": polygons,
+            "bbox": self.__get_box_from_coords(bbox),
             "color": color,
         })
 
@@ -534,39 +694,53 @@ class Images(Entities):
 
         return imgs_return
 
-    def __draw_text_with_shadow(self, image, text, origin, color, typeface=cv2.FONT_HERSHEY_SIMPLEX, scale=0.75, thickness=2, shadow_color=None, shadow_radius=4):
+    def __draw_text_with_shadow(self, image, text, origin, color, typeface=cv2.FONT_HERSHEY_SIMPLEX, scale=0.75, thickness=2, shadow_color=None, shadow_radius=4, meta=None):
+        width, height = image.shape[1], image.shape[0]
+        thickness = max(1, int(min(width, height) / 200))
         if not shadow_color:
             shadow_color = self.__contrasting_color(color)
 
-        cv2.putText(image, text, origin, typeface, scale,
+        scale = thickness / 3
+        resolved_origin = np.array([origin])
+        operations = self.query["operations"] if self.query and "operations" in self.query else [
+        ]
+        if meta and "adb_image_width" in meta and "adb_image_height" in meta:
+            resolved_origin = resolve(resolved_origin, meta, operations)
+        else:
+            logger.warning(
+                f"Cannot resolve text origin, missing image metadata in {meta}.")
+        cv2.putText(image, text, resolved_origin[0], typeface, scale,
                     shadow_color, thickness + shadow_radius, cv2.LINE_AA)
 
-        cv2.putText(image, text, origin, typeface,
+        cv2.putText(image, text, resolved_origin[0], typeface,
                     scale, color, thickness, cv2.LINE_AA)
 
-    def __draw_bbox(self, image, bbox, color, thickness=2):
+    def __draw_bbox(self, image, bbox, color, thickness=1):
+        width, height = image.shape[1], image.shape[0]
+        thickness = max(1, int(min(width, height) / 200))
+        cv2.drawContours(image, [bbox], 0, color, thickness)
 
-        left   = bbox["x"]
-        top    = bbox["y"]
-        right  = bbox["x"] + bbox["width"]
-        bottom = bbox["y"] + bbox["height"]
-        cv2.rectangle(image, (left, top), (right, bottom), color, thickness)
+    def __get_color_for_tag(self, tag: str) -> Tuple[int, int, int]:
+        if tag not in self.color_for_tag:
+            color = self.__random_color()
+            self.color_for_tag[tag] = color
+        return self.color_for_tag[tag]
 
-    def __draw_bbox_and_tag(self, image, bbox, tag, deferred_tags):
-        RED = (0, 0, 255)
-
-        self.__draw_bbox(image, bbox, RED)
+    def __draw_bbox_and_tag(self, image, bbox, tag, bound, meta, deferred_tags):
+        color = self.__get_color_for_tag(tag)
+        self.__draw_bbox(image, bbox, color)
 
         if tag:
-            left   = bbox["x"]
-            top    = bbox["y"]
+            left = bound[0][0]
+            top = bound[0][1]
 
             y = top - 15 if top - 15 > 15 else top + 15
 
             deferred_tags.append({
                 "text": tag,
                 "origin": (left, y),
-                "color": RED
+                "color": color,
+                "meta": meta
             })
 
     def __random_color(self, value=1.0, saturation=0.51):
@@ -592,28 +766,28 @@ class Images(Entities):
         return (desaturate(cont[0]), desaturate(cont[1]), desaturate(cont[2]))
 
     def __draw_polygon(self, image, polygon, color, fill_alpha=0.4, thickness=2, shift=8):
-
-        def as_shift_int(v): return int(v * (1 << shift))
+        width, height = image.shape[1], image.shape[0]
+        thickness = max(1, int(min(width, height) / 200))
 
         for verts in polygon:
-            shift_verts = [[as_shift_int(v) for v in vert] for vert in verts]
-            np_verts = np.array(shift_verts, np.int32)
+            np_verts = np.multiply(verts, 256)
             fill = image.copy()
-            cv2.fillPoly(fill, [np_verts], color, cv2.LINE_4, shift)
+            cv2.fillPoly(fill, [np_verts.astype(int)],
+                         color, cv2.LINE_4, shift)
             cv2.addWeighted(fill, fill_alpha, image, 1 - fill_alpha, 0, image)
-            cv2.polylines(image, [np_verts], True, color,
+            cv2.polylines(image, [np_verts.astype(int)], True, color,
                           thickness, cv2.LINE_4, shift)
 
-    def __draw_polygon_and_tag(self, image, polygon, tag, bounds, deferred_tags):
+    def __draw_polygon_and_tag(self, image, polygon, tag, bounds, meta, deferred_tags):
 
-        color = self.__random_color()
+        color = self.__get_color_for_tag(tag)
 
         self.__draw_polygon(image, polygon, color)
 
         if tag:
-            left   = bounds["x"]
-            top    = bounds["y"]
-            right  = bounds["x"] + bounds["width"]
+            left = bounds["x"]
+            top = bounds["y"]
+            right = bounds["x"] + bounds["width"]
             FONT_WIDTH = 10
             FONT_HEIGHT = 16
 
@@ -624,11 +798,37 @@ class Images(Entities):
             deferred_tags.append({
                 "text": tag,
                 "origin": org,
-                "color": color
+                "color": color,
+                "meta": meta
             })
 
-    def display(self, show_bboxes=False, show_polygons=False, limit=None, polygon_constraints=None, polygon_tag_key="_label", polygon_tag_format="{}"):
+    def display(self,
+                show_bboxes: bool = False,
+                bbox_constraints: Constraints = None,
+                show_polygons: bool = False,
+                limit: Union[int, object] = None,
+                polygon_constraints: Constraints = None,
+                polygon_tag_key: str = "_label",
+                polygon_tag_format: str = "{}") -> None:
+        """
+        **Display images with annotations**
 
+        Args:
+            show_bboxes: bool, optional
+                Show bounding boxes, by default False
+            bbox_constraints: Constraints, optional
+                Constraints for bounding boxes, by default None
+            show_polygons: bool, optional
+                Show polygons, by default False
+            limit: Union[int, object], optional
+                Number of images to display, by default None
+            polygon_constraints: Constraints, optional
+                Constraints for polygons, by default None
+            polygon_tag_key: str, optional
+                Key for the polygon tag, by default "_label"
+            polygon_tag_format: str, optional
+                Format for the polygon tag, by default "{}"
+        """
         if not limit:
             limit = self.display_limit
             if self.display_limit < len(self.images_ids):
@@ -645,7 +845,7 @@ class Images(Entities):
             limit -= 1
 
             uniqueid = str(self.images_ids[i])
-            image    = self.get_image_by_index(i)
+            image = self.get_image_by_index(i)
 
             # Just decode the image from buffer
             nparr = np.frombuffer(image, dtype=np.uint8)
@@ -654,15 +854,17 @@ class Images(Entities):
             deferred_tags = []
             if show_bboxes:
                 if not str(uniqueid) in self.images_bboxes:
-                    self.__retrieve_bounding_boxes(i)
+                    self.__retrieve_bounding_boxes(i, bbox_constraints)
 
                 bboxes = self.images_bboxes[uniqueid]["bboxes"]
-                tags   = self.images_bboxes[uniqueid]["tags"]
+                tags = self.images_bboxes[uniqueid]["tags"]
+                meta = self.images_bboxes[uniqueid]["meta"]
+                bounds = self.images_bboxes[uniqueid]["bounds"]
 
                 # Draw a rectangle around the faces
                 for bi in range(len(bboxes)):
                     self.__draw_bbox_and_tag(
-                        image, bboxes[bi], tags[bi], deferred_tags)
+                        image, bboxes[bi], tags[bi], bounds[bi], meta[bi], deferred_tags)
 
             if show_polygons:
                 if str(uniqueid) not in self.images_polygons:
@@ -675,10 +877,12 @@ class Images(Entities):
                 ]
                 tags = self.images_polygons[uniqueid]["tags"] if uniqueid in self.images_polygons else [
                 ]
+                meta = self.images_polygons[uniqueid]["meta"] if uniqueid in self.images_polygons else [
+                ]
 
                 for pi in range(len(polygons)):
                     self.__draw_polygon_and_tag(image, polygons[pi], tags[pi] if pi < len(
-                        tags) else None, bounds[pi], deferred_tags)
+                        tags) else None, bounds[pi], meta[pi], deferred_tags)
 
             for ovr in self.overlays:
                 if "polygons" in ovr:
@@ -691,15 +895,20 @@ class Images(Entities):
 
             for tag in deferred_tags:
                 self.__draw_text_with_shadow(
-                    image, tag["text"], tag["origin"], tag["color"])
+                    image, tag["text"], tag["origin"], tag["color"], meta=tag["meta"] if "meta" in tag else None)
 
             image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
 
             fig1, ax1 = plt.subplots()
             plt.imshow(image)
 
-    def get_props_names(self):
+    def get_props_names(self) -> List[str]:
+        """
+        **Get the names of the properties that apply to images**
 
+        Returns:
+            properties (List[str]): The names of the properties of the images
+        """
         dbutils = Utils.Utils(self.db_connector)
         schema = dbutils.get_schema()
 
@@ -712,8 +921,16 @@ class Images(Entities):
 
         return props_array
 
-    def get_properties(self, prop_list=[]):
+    def get_properties(self, prop_list: Iterable[str] = []) -> Dict[str, Any]:
+        """
+        **Get the properties of the images**
 
+        Args:
+            prop_list (List[str], optional): The list of properties to retrieve. Defaults to [].
+
+            Returns:
+                property_values Dict[str, Any]: The properties of the images
+        """
         if len(prop_list) == 0:
             return {}
 
