@@ -15,7 +15,7 @@ class PyTorchData(Subscriptable):
     This class should be subclassed with a specific (custom) implementation of generate_query().
     :::
 
-    Example subclass: [CocoDataPytorch](https://github.com/aperture-data/aperturedb-python/blob/develop/examples/CocoDataPytorch.py)
+    Example subclass: [CocoDataPyTorch](https://github.com/aperture-data/aperturedb-python/blob/develop/examples/CocoDataPyTorch.py)
 
     """
 
