@@ -4,8 +4,8 @@ class Stats:
     total_actions_time = 0
     error_counter = 0
     objects_existed  = 0
-    suceeded_queries = 0
-    suceeded_commands = 0
+    succeeded_queries = 0
+    succeeded_commands = 0
 
     def __init__(self):
         self.total_actions = 0
@@ -13,5 +13,5 @@ class Stats:
         self.total_actions_time = 0
         self.error_counter = 0
         self.objects_existed = 0
-        self.suceeded_queries = 0
-        self.suceeded_commands = 0
+        self.succeeded_queries = 0
+        self.succeeded_commands = 0
