@@ -16,11 +16,13 @@ except ImportError:
     logger.critical(error_message)
     exit(1)
 
+device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
+
 # If required, create a face detection pipeline using MTCNN:
-mtcnn = MTCNN(image_size=96, margin=0)
+mtcnn = MTCNN(image_size=96, margin=0, device=device)
 
 # Create an inception resnet (in eval mode):
-resnet = InceptionResnetV1(pretrained='vggface2').eval()
+resnet = InceptionResnetV1(pretrained='vggface2', device=device).eval()
 
 errors = 0
 
@@ -31,9 +33,9 @@ def generate_embedding(img):
     img_cropped = mtcnn(img)
     if img_cropped is not None:
         # Calculate embedding (unsqueeze to add batch dimension)
-        img_embedding = resnet(img_cropped.unsqueeze(0))
+        img_embedding = resnet(img_cropped.unsqueeze(0).to(device))
     else:
-        img_embedding = torch.FloatTensor(1, 512)
+        img_embedding = torch.FloatTensor(1, 512).to(device)
         errors += 1
 
     return img_embedding
